// micro-module to test functionality
(function() {
	'use strict';

  var hello = function () {
    return "hello";
  }

  module.exports = hello;

}()); // benalman.com/news/2010/11/immediately-invoked-function-expression
