"use strict";

var lenientProperties = ["entries", "sort", "values"];
var strictProperties = ["append", "delete", "get", "getAll", "has", "keys", "set"];
module.exports = {
  lenientProperties: lenientProperties,
  strictProperties: strictProperties
};
//# sourceMappingURL=props.js.map